#include <sys/file.h>
#include <sys/errno.h>
#include <string.h>
#include <stdio.h>
#include <math.h>

#define R_OK 04
#define F_OK 00
#define O_RDWR 06

struct datafilehead
{
    long nblocks;        /* number of blocks in file     */
    long ntraces;        /* number of traces per block   */
    long np;             /* number of elements per trace */
    long ebytes;         /* number of bytes per element  */
    long tbytes;         /* number of bytes per trace    */
    long bbytes;         /* number of bytes per block    */
    short transf;        /* transposed storage flag      */
    short status;        /* status of whole file         */
    long spare1;         /* reserved for future use      */
} main_header;    

struct datablockhead
{
    short scale;         /* scaling factor               */
    short status;        /* status of data in block      */
    short index;         /* block index                  */
    short spare3;        /* reserved for future use      */
    long  ctcount;       /* completed transients in fids */
    float lpval;         /* left phase in phasefile      */
    float rpval;         /* right phase in phasefile     */
    float lvl;           /* level drift correction       */
    float tlt;           /* tilt drift correction        */
} block_header;

float dphi(float x1,float x2,float y1,float y2)
    {
    float iph;
    iph = (float)(5729.57795e-6*atan2((x1*y2-x2*y1),(x1*x2+y1*y2)));
    return iph ;
    } 
    /*dphi*/

void getphase(int si, int idx, int wd2, float *wbuff1, float *wbuff, int ni, int nf)
{
  int j;
  float n11, n12, n21, n22, m1, m2;
  
  n11=0.0;n12=0.0;
  n21=0.0;n22=0.0;
  for (j=0; j<si/16; j++)
  {
    n11 += wbuff[ni*si+2*j]*wbuff[ni*si+2*j]+wbuff[ni*si+2*j+1]*wbuff[ni*si+2*j+1];
    n12 += wbuff[nf*si+2*j]*wbuff[nf*si+2*j]+wbuff[nf*si+2*j+1]*wbuff[nf*si+2*j+1];
  }
  for (j=7*si/16; j<si/2; j++)
  {
    n21 += wbuff[ni*si+2*j]*wbuff[ni*si+2*j]+wbuff[ni*si+2*j+1]*wbuff[ni*si+2*j+1];
    n22 += wbuff[nf*si+2*j]*wbuff[nf*si+2*j]+wbuff[nf*si+2*j+1]*wbuff[nf*si+2*j+1];
  }
  if (n12>n11)
  {
    n12 = n11;
  }
  else
  {
    n11 = n12;
  }
  if (n22>n21)
  {
    n22 = n21;
  }
  else
  {
    n21 = n22;
  }
  if (n22>n11)
  {
    n22 = n11;
  }
  else
  {
    n11 = n22;
  }
  n11 = n11*16/si;
/*end determination of RMS noise level */
  for (j=si/4-wd2-1; j<si/4+wd2+1; j++)
  {
    wbuff1[idx*si+2*j] = dphi(wbuff[ni*si+2*j],wbuff[nf*si+2*j],wbuff[ni*si+2*j+1],wbuff[nf*si+2*j+1]);
    m1 = wbuff[ni*si+2*j]*wbuff[ni*si+2*j]+wbuff[ni*si+2*j+1]*wbuff[ni*si+2*j+1];
    m2 = wbuff[nf*si+2*j]*wbuff[nf*si+2*j]+wbuff[nf*si+2*j+1]*wbuff[nf*si+2*j+1];
    if ((m1>=100*n11) && (m2>=100*n11))
    {
      wbuff1[idx*si+2*j+1] = 1/m2+1/m1;
    }
    else
    {
      wbuff1[idx*si+2*j+1] = 0.0;
    }
  }
/*
wbuff[i*np+2*j]=dphi((float)in_data[2*i].lo[2*j],(float)in_data[2*i+1].lo[2*j],(float)in_data[2*i].lo[2*j+1],(float)in_data[2*i+1].lo[2*j+1]);
*/
}

void getdata(int *si, int *nproj, float *wbuff, char infile_name1[])
    
{
    long    element_size, maxval=16384, scale_factor, temp;
    int     i, j, k,l, nfids_in, in_file, np;
    float   stor, max;
    FILE    *in_ptr;
    char    string[100];
    char    *malloc();
    union   precision {
		short  *sh;
		float   *lo;
	    } *in_data;


    
    /****
    * Determine the accessibility of the input directory.    ****/

    if ((access(infile_name1, F_OK | R_OK))) 
    {
	perror("Access");
	printf("Input file either does not exist, or cannot be read.\n");
	exit(2);
    }

    /****
    * Open input fid file.
    ****/
    strcpy(string, infile_name1);
    in_file = open(string, O_RDWR);
    if (in_file < 0) 
    {
	perror("Open");
	printf("Can't open input fid file.\n");
	exit(4);
    }

    /****
    * Read main_header at top of input file.
    ****/
    if (read(in_file, &main_header, 32) != 32) 
    {
	perror ("read");
	exit(5);
    }
    np = main_header.np;
    *si=np;
    element_size = main_header.ebytes;
    nfids_in = main_header.nblocks;
    *nproj=nfids_in;
 
    /****
    * Allocate space for input data.  All calculations will be done
    * with "long" integers (normal "int" on the SUN), so wee will
    * allocate space for this, even if the data is "short".
    ****/
    in_data = (union precision *)malloc(nfids_in*sizeof(int*));
    for (i=0; i<nfids_in; i++) 
    {
	in_data[i].lo = (float *)malloc(np*sizeof(float));    
    }
    /****
    * Read in raw data.
    ****/
    for (i=0; i<nfids_in; i++) 
    {
    if (read(in_file, &block_header, 28) != 28) 
    {
	perror ("read2");
	exit(5);
    }
    if (read(in_file, in_data[i].lo, np*sizeof(float)) != np*sizeof(float)) 
    {
	perror ("read3");
	exit(5);
    }
    }
    for (i=0; i<nfids_in; i++)
      {
      for (j=0; j<np; j++)
      {
       wbuff[i*np+j]=(float)in_data[i].lo[j];
      }
      }
    fprintf(stderr,"done reading .. ");
    fprintf(stderr,"%5d points in %3d projections (%2d el. size)\n",*si,nfids_in,element_size);
    close(in_file);
    free(in_data);
  }/*getdata*/


void putdata(int *si, float *wbuff, char infile_name1[])
    
{
    long    element_size, maxval=16384, scale_factor, temp;
    int     i, j, k,l, nfids_out, out_file, np;
    float   stor, max;
    FILE    *out_ptr;
    char    string[100];
    char    *malloc();
    union   precision {
		short  *sh;
		float   *lo;
	    } *out_data;

    strcpy(string, infile_name1);
    out_file = open(string, O_RDWR);
    if (out_file < 0) 
    {
	perror("Open");
	printf("Can't open output fid file.\n");
	exit(4);
    }
    np = main_header.np;
    nfids_out = main_header.nblocks;
 
    /****
    * Allocate space for output data.  All calculations will be done
    * with "long" integers (normal "int" on the SUN), so wee will
    * allocate space for this, even if the data is "short".
    ****/
    out_data = (union precision *)malloc(nfids_out*sizeof(int*));
    for (i=0; i<nfids_out; i++) 
    {
	out_data[i].lo = (float *)malloc(np*sizeof(float));    
    }
    write(out_file, &main_header, 32);

    for (i=0; i<nfids_out; i++)
      {
      for (j=0; j<np/2-1; j++)
      {
	out_data[i].lo[2*j] = wbuff[i*np+2*j];
	out_data[i].lo[2*j+1] = wbuff[i*np+2*j+1];
      }
    }
    for (k=0; k<nfids_out; k++) {
	block_header.index = k + 1;
	write(out_file, &block_header, 28);
	write(out_file, out_data[k].lo, np*sizeof(float));
    }
    close(out_file);
    free(out_data);
}
